<?php
require __DIR__ . '/../vendor/autoload.php';

use Psr\Http\Message\ResponseInterface as Response;
use Psr\Http\Message\ServerRequestInterface as Request;
use Slim\Factory\AppFactory;
use InfluxDB2\Client;
use InfluxDB2\Model\WritePrecision;
use InfluxDB2\Point;

/**
 * Instantiate App
 *
 * In order for the factory to work you need to ensure you have installed
 * a supported PSR-7 implementation of your choice e.g.: Slim PSR-7 and a supported
 * ServerRequest creator (included with Slim PSR-7)
 */
$app = AppFactory::create();

/**
 * The routing middleware should be added earlier than the ErrorMiddleware
 * Otherwise exceptions thrown from it will not be handled by the middleware
 */
$app->addRoutingMiddleware();

/**
 * Add Error Middleware
 *
 * @param bool $displayErrorDetails
 *            -> Should be set to false in production
 * @param bool $logErrors
 *            -> Parameter is passed to the default ErrorHandler
 * @param bool $logErrorDetails
 *            -> Display error details in error log
 * @param LoggerInterface|null $logger
 *            -> Optional PSR-3 Logger
 *            
 *            Note: This middleware should be added last. It will not handle any exceptions/errors
 *            for middleware added after it.
 */
$errorMiddleware = $app->addErrorMiddleware(true, true, true);

$app->setBasePath('/central');

/*
 * route GET/POST permettant l'affichage de la page central, avec l'intégration si besoin de toutes les pages admin appellées.
 */
$app->map([
    'GET',
    'POST'
], '/{page}', function (Request $request, Response $response, $args) {
    include "config.inc.php";
    require "ldap.inc.php";
    require_once ("functions.inc.php");
    require_once ("traitement_data.inc.php");
    $config = get_config();
    include 'admin_ui.inc.php';
    include "ihm.inc.php";
    include 'annu.inc.php';
    include 'parcs.inc.php';
    require_once "fonc_outils.inc.php";
    require_once "fonc_parc.inc.php";
    require_once "logs.inc.php";
    require_once ("remote.inc.php");
    include 'sites.inc.php';
    include 'ent.inc.php';
    // reecriture du nom de la page appelée en fonction (par défaut index.php)
    $html_page = preg_replace("#\.php#", "_html", $args['page']);
    // ajouter ici les truc de index

    /*
     * Includes du central
     */
    include ("conf.inc.php");
    include ("auth.inc.php");
    include ("stats_serveurs_se4.inc.php");
    include ("stats_serveurs_pve.inc.php");
    include ("central_ui.inc.php");
    include ("alertes.inc.php");
    include ("tdb_ui.inc.php");
    include ("annu_ui.inc.php");
    include ("auns_ui.inc.php");
    include ("eds_ui.inc.php");
    include ("sdpve_ui.inc.php");
    include ("sds_ui.inc.php");
    include ("tabstats_ui.inc.php");
    include ("dda_ui.inc.php");
    require __DIR__ . '/../vendor/autoload.php';

    /*
     *
     */

    if ((isset($_SESSION['login'])) && (have_right($config, SE_ADMIN, $_SESSION['login'])) || (have_right($config, SE_USER_ADMIN, $_SESSION['login']))) {
        $_SESSION['username'] = $_SESSION['login'];
    }

    if (! isset($_SESSION['username'])) {
        if (isset($_POST['username']) && isset($_POST['password'])) {
            if (auth($_POST['username'], $_POST['password'])) {
                $_SESSION['username'] = $_POST['username'];
                $_SESSION['login'] = $_SESSION['username'];
                $config['login'] = $_SESSION['username'];
            } else {
                login($config, $_SERVER['REQUEST_URI']);
            }
        } else {
            login($config, $_SERVER['REQUEST_URI']);
        }
    }

    /*
     * Par défaut on charge le tableau de bord
     * Et si un host a été envoyé, on vérifie qu'il est bien associé à un établissement
     * sinon on vide le host et on renvoie la page
     * Voir si on ajoute un message d'erreur... en variable de session par exemple qui s'affiche avant toute chose.
     */
    $page = $_GET['page'] ?? "tdb";
    $etab = $_POST['etab'] ?? $_GET['etab'] ?? $_POST['host'] ?? $_GET['host'] ?? $_SESSION['etab'] ?? "";
    if (! empty($etab)) {
        $name_etab = etab_to_name(get_config($config, true, $etab), $etab) ?? "";
        if (empty($name_etab)) {
            $etab = $_POST['etab'] = $_GET['etab'] = $_POST['host'] = $_GET['host'] = $_SESSION['etab'] = "";
            /**
             * Erreur de saisie de UAI
             * on stocke un message d'erreur dans la
             * variable de session
             */
            $_SESSION['erreur'] = "UAI";
        } else {
            $_SESSION['erreur'] = "";
        }
    } else {
        $_SESSION['erreur'] = "";
    }
    $_SESSION['etab'] = $etab;
    /*
     * mise à jour du geojson avec les infos de CPU/RAM
     */
    update_servers_conf();
    /*
     * On charge la liste des alertes
     */
    liste_alertes($config);

    if ($html_page == "index_html") {
        /*
         * Début de la page (header, topbar, menu, wrapper)
         */
        $html = html_top($config, $page);
        $html .= html_content($config, $page);
    } else {
        $page = "";
        $html = html_top($config, $page);
        /**
         * *
         * on encapsule la page envoyée par la fonction
         * dans un cadre bootstraconfig.inc.phpp avec un titre
         * si le cn est non vide, on affiche le formulaire de recherche
         * ainsi qu'un lien vers la fiche user
         */
        $CadreTitre = "$name_etab";
        if (! empty($_GET['cn'])) {
            // annu_central($config, $html);
            $CadreTitre .= " - Retour sur la fiche de : <a href=\"index.php?page=r&cn=" . $_GET['cn'] . "\">" . $_GET['cn'] . "</a>";
        }
        $html .= '<div class="row">
						<div class="col-xl-12 col-lg-7 ">
							<div class=" card shadow mb-4">
								<div
									class="card-header py-3 d-flex flex-row align-items-center justify-content-between">
									<h6 class="m-0 font-weight-bold text-primary">' . $CadreTitre . '</h6>
								</div>
								<div class="card-body ">';
        ($html_page)(get_config($config, true, $etab), $html);
        $html .= '				</div>
							</div>
						</div>
					</div>';
    }
    /*
     * footer
     */
    $html .= html_footer($config, $page);

    $response->getBody()
        ->write($html);
    return $response;
});

// Run app
$app->run();

?>