<?php
/*
 * Mise à jour des alertes
 */
function alertes($config)
{
    $servers_conf = load_servers_conf();
    $allLastStats = allLastStats($config);
    $allStatsPVE = allLastStatsPVE($config);
    $allLastVM = allLastStatsVM($config);

    /*
     * init du tableau des alertes
     */
    $alertes["alive"] = [];
    $alertes["dskhome"] = [];
    $alertes["dskpve"] = [];

    $success = false;
    $alertes = apcu_fetch('alertes', $success);
    if (!$success) {
        foreach ($servers_conf as $cle => $valeur) {
            $uai = strtolower($valeur['fields']['uai']);
            /*
             * Serveurs injoignables
             */
            $etat = $servers_conf[$cle]['fields']['etat'] ?? "";
            if ("$etat" == "injoignable") {
                $since = (time() - $servers_conf[$cle]['fields']['alive']);
                if ($since < 120) {
                    $depuis = "$since secondes";
                } elseif ($since < 3600) {
                    $since = intval($since / (60)) + 1;
                    $depuis = "$since  minutes";
                } elseif ($since < 3600 * 24) {
                    $since = intval($since / (3600)) + 1;
                    $depuis = "$since heures";
                } else {
                    $since = intval($since / (3600 * 24)) + 1;
                    $depuis = "$since jours";
                }
                $alertes["alive"]["$uai"] = "Le serveur SE4FS $uai est injoignable depuis $depuis.";
            }
            /*
             * home pleins à plus de 80%
             */
            if ((key_exists($uai, $allLastStats)) && (key_exists('home', $allLastStats[$uai])) && ($allLastStats[$uai]['home'] > 80)) {
                $alertes["dskhome"]["$uai"] = "Le disque '/home' du serveur SE4FS $uai  est plein à " . $allLastStats[$uai]['home'] . "%.";
            }
            /*
             * Espace du PVE
             */
            if (isset($allStatsPVE["pve-" . $uai])) {
                $diskTotal = $allStatsPVE["pve-" . $uai]['blocks'];
                $diskUsed = $allStatsPVE["pve-" . $uai]['used'];
                if (($diskTotal != 0) && ($diskTotal != "")) {
                    $dskPVEPRC = round($diskUsed / $diskTotal * 100, 2);
                    if ($dskPVEPRC > 60) {
                        $alertes["dskpve"]["$uai"] = "Le disque du serveur PVE de $uai est plein à " . $dskPVEPRC . "%.";
                    }
                }
            }
            /*
             *
             * AD à plus de 60% de CPU
             *
             */
            if (isset($allLastVM["se4ad-" . $uai])) {
                $CPUAD = round(100 * $allLastVM['se4ad-' . $uai]['cpu'], 2);
                if ($CPUAD > 60) {
                    $alertes["cpuad"]["$uai"] = "L'utilisation de la CPU du serveur AD de $uai est supérieure à " . $CPUAD . "%.";
                }
            }
        }
        apcu_add('alertes', $alertes, 300);
    }
    return $alertes;
}

/*
 * Fabrique le bandeau déroulant des alertes
 */
function liste_alertes($config)
{
    $success = false;
    $liste_alertes = apcu_fetch('liste_alertes', $success);
    if (!$success) {
        $liste_alertes = "";
        $alertes = alertes($config);
        if (key_exists("alive", $alertes)) {
            foreach ($alertes["alive"] as $cle => $valeur) {
                $liste_alertes .= item_alert_ui($valeur, $cle, "fa-solid fa-heart-crack");
            }
        }
        if (key_exists("dskhome", $alertes)) {
            foreach ($alertes["dskhome"] as $cle => $valeur) {
                $liste_alertes .= item_alert_ui($valeur, $cle, "fa-regular fa-floppy-disk");
            }
        }
        if (key_exists("dskpve", $alertes)) {
            foreach ($alertes["dskpve"] as $cle => $valeur) {
                $liste_alertes .= item_alert_ui($valeur, $cle, "fa-solid fa-hard-drive");
            }
        }
        if (key_exists("cpuad", $alertes)) {
            foreach ($alertes["cpuad"] as $cle => $valeur) {
                $liste_alertes .= item_alert_ui($valeur, $cle, "fa-solid fa-microchip");
            }
        }
        apcu_add('liste_alertes', $liste_alertes, 300);
    }
    return $liste_alertes;
}

function item_alert_ui($valeur, $cle, $icon = "fas fa-file-alt")
{
    return '<a class="dropdown-item d-flex align-items-center" href="?page=sds&host=' . $cle . '">
									<div class="mr-3">
									<div class="icon-circle bg-danger">
										<i class="' . $icon . ' text-white"></i>
									</div>
									</div>
									<div>
										<span class="font-weight-bold">
											' . $valeur . '
										</span>
									</div>
									</a>';
}
