<?php
/**
 * Query influxdb
 */
require_once ('/var/www/sambaedu/vendor/autoload.php');
use InfluxDB2\Client;
use InfluxDB2\Model\WritePrecision;
use InfluxDB2\Point;
use Proxmox\PVE;
use Proxmox\Api\Nodes\Node\Storage\Storage;

/*
 * Retourne les derniers enregistrements
 *
 */
function StatsServeurPVE($config, $quoi = "cpu", $host = "all", $max = false)
{
    $start = "-5d";
    if ($quoi == "RAM") {
        $measure = "memused";
        $field = "memused";
    } else if ($quoi == "CPU") {
        $measure = "cpu";
        $field = "cpu";
    } else if ($quoi == "nics") {
        $measure = "nics";
        $field = "receive";
        // $instance = "vmbr0";
    } else {
        $measure = "memused";
        $field = "memused";
    }
    return InfluxQueryPVE($config, $measure, $field, $host, $start, false);
}

function InfluxQueryPVE($config, $measure, $field, $host, $start = "-1d", $last = false, $max = false)
{
    if ($max) {
        $maxquery = '|> max()';
    } else {
        $maxquery = '';
    }
    if ($last) {
        $lastq = "|> last()";
        $movingAverage = "";
        $aggregate = "";
    } else {
        $lastq = "";
        $duree = "10m";
        $average = 10;
        $movingAverage = "|> movingAverage(n:" . $average . ")";
        $aggregate = "|> aggregateWindow(every: " . $duree . ", fn: mean, createEmpty: false)";
    }

    $client = new Client([
        "url" => $config['influxdb_url'],
        "token" => $config['influxdb_token']
    ]);

    $queryApi = $client->createQueryApi();

    if ($measure == "memused") {
        $query = "from(bucket: \"" . $config['influxdb_bucket'] . "\")
    |> range(start: " . $start . ")
    " . $lastq . "
    |> filter(fn: (r) => r[\"_field\"] == \"memused\" or r[\"_field\"] == \"memtotal\")
    |> filter(fn: (r) => r[\"host\"] == \"$host\")
    " . $aggregate . "
    |> pivot(rowKey: [\"_time\"], columnKey: [\"_field\"], valueColumn: \"_value\")
    |> map(
    fn: (r) => ({
        r with _value: float(v: r.memused) / float(v: r.memtotal) * 100.0
        }),
        )
    " . $movingAverage . "
    |> yield(name: \"mean\")";
    } elseif ($measure == "cpu") {
        $fields = "|> filter(fn: (r) => r[\"_field\"] == \"$field\")
					|> filter(fn: (r) => r[\"_measurement\"] == \"cpustat\")";
        $query = "from(bucket: \"" . $config['influxdb_bucket'] . "\")
    |> range(start: " . $start . ")
    " . $lastq . "
    " . $fields . "
    " . $aggregate . "
    |> filter(fn: (r) => r[\"host\"] == \"$host\")
    |> map(fn: (r) => ({r with _value: r._value * 100.0}))
    " . $movingAverage . "
    |> yield(name: \"mean\")";
    } elseif ($measure == "nics") {
        $fields = "|> filter(fn: (r) => r[\"_field\"] == \"$field\")";
        $query = "from(bucket: \"" . $config['influxdb_bucket'] . "\")
    |> range(start: " . $start . ")
    " . $lastq . "
    " . $fields . "
    " . $aggregate . "
    |> filter(fn: (r) => r[\"instance\"] == \"vmbr0\")
    |> filter(fn: (r) => r[\"host\"] == \"$host\")
    |> map(fn: (r) => ({r with _value: r._value / 1000.0}))
	|> increase()
    |> difference()
    " . $movingAverage . "
    " . $maxquery . "
        
    ";
    }

    if ($last || $max) {
        $tables = $queryApi->query($query, $config['influxdb_org']);
        foreach ($tables as $table) {
            foreach ($table->records as $record) {
                if (isset($record->values) && is_object($record->values)) {
                    $records[$record] = $record->values;
                }
            }
        }
        if (isset($record)) {
            $value = round($record->values['_value'], 2) ?? "0";
            $time = $record->values['_time'] ?? "0";
            return [
                $value,
                $time
            ];
        } else {
            return [
                0,
                0
            ];
        }
    } else {
        $records = $queryApi->queryRaw($query, $config['influxdb_org']);
        return $records;
    }
}

/*
 * Retourne un tableau avec les derniers enregistrements du PBS
 */
function allLastStatsPBS($config)
{
    $success = false;
    $last = apcu_fetch('allLastStatsPBS', $success);
    if (! $success) {

        $client = new Client([
            "url" => $config['influxdb_url'],
            "token" => $config['influxdb_token']
        ]);
        $queryApi = $client->createQueryApi();

        $start = "-10m";
        $query = "from(bucket: \"" . $config['influxdb_bucket'] . "\")
    |> range(start: " . $start . ")
    |> last()
    |> filter(fn: (r) => r[\"host\"] == \"" . $config['pbs_host'] . "\")
    |> filter(fn: (r) => r[\"datastore\"] == \"" . $config['pbs_datastore'] . "\")
    |> filter(fn: (r) => r[\"_measurement\"] == \"blockstat\")
    |> yield(name: \"mean\")";

        $tables = $queryApi->query($query, $config['influxdb_org']);

        $last = array();
        foreach ($tables as $table) {
            foreach ($table->records as $record) {
                $host = $record->values['host'];
                $last[$host][$record->values['_field']] = $record->values['_value'];
            }
        }
        apcu_add('allLastStatsPBS', $last, 3600);
    }

    return $last;
}

/*
 * Retourne un tableau avec les derniers enregistrements du PVE
 */
function allLastStatsPVE($config)
{
    $success = false;
    $last = apcu_fetch('allLastStatsPVE', $success);
    // $success = false;
    if (! $success) {

        $client = new Client([
            "url" => $config['influxdb_url'],
            "token" => $config['influxdb_token']
        ]);
        $queryApi = $client->createQueryApi();

        $start = "-10m";
        $query = "from(bucket: \"" . $config['influxdb_bucket'] . "\")
    |> range(start: " . $start . ")
    |> last()
    |> filter(fn: (r) => r[\"_measurement\"] == \"blockstat\" or r[\"_measurement\"] == \"memory\" or r[\"_measurement\"] == \"cpustat\")
    |> filter(fn: (r) => r[\"_field\"] == \"used\" or r[\"_field\"] == \"memused\"  or r[\"_field\"] == \"memtotal\" or r[\"_field\"] == \"blocks\" or r[\"_field\"] == \"cpu\" )
    |> yield(name: \"mean\")";

        $tables = $queryApi->query($query, $config['influxdb_org']);

        $last = array();
        foreach ($tables as $table) {
            foreach ($table->records as $record) {
                $host = $record->values['host'];
                $last[$host][$record->values['_field']] = $record->values['_value'];
            }
        }
        apcu_add('allLastStatsPVE', $last, 500);
    }

    return $last;
}

/*
 * Retourne un tableau avec les derniers stats des VM du PVE
 */
function allLastStatsVM($config)
{
    $success = false;
    $last = apcu_fetch('allLastStatsVM', $success);
    // $success = false;
    if (! $success) {

        $client = new Client([
            "url" => $config['influxdb_url'],
            "token" => $config['influxdb_token']
        ]);

        $queryApi = $client->createQueryApi();

        $start = "-10m";
        $query = "from(bucket: \"" . $config['influxdb_bucket'] . "\")
    |> range(start: " . $start . ")
    |> last()
    |> filter(fn: (r) => r[\"_field\"] == \"cpu\")
    |> yield(name: \"mean\")";

        $tables = $queryApi->query($query, $config['influxdb_org']);

        $last = array();
        foreach ($tables as $table) {
            foreach ($table->records as $record) {
                $host = $record->values['host'];
                $last[$host][$record->values['_field']] = $record->values['_value'];
            }
        }
        apcu_add('allLastStatsVM', $last, 500);
    }

    return $last;
}

/*
 * On utilise l'API composer require mrkampf/proxmox-ve
 * pour récupérer les info avec l'PI PVE
 *
 */
function countPveNodesRunning($config, $uai)
{
    $PVE_IP = PVE_IP_DepuisUAI($uai);
    if ($PVE_IP) {
        /*
         * TODO
         * UTILISER une clé API !
         */

        $proxmox = new PVE("$PVE_IP", "root", $config['pve_password'], 8006, "pam", false);
        $NB_QEMU = ($proxmox->nodes()
            ->node("pve-$uai")
            ->qemu()
            ->get()['data']);
        $result = array_filter($NB_QEMU, function ($v) {
            return $v['status'] == "running";
        });
        return (count($result));
    } else {
        return "---";
    }
}

/*
 * Retourne le nombre de SVG du sE4FS
 * depuis l'UAI
 * ainsi que la date de la dernière
 * ainsi que sa taille
 */
function Sauvegardes($config, $uai)
{
    $PVE_IP = PVE_IP_DepuisUAI($uai);
    // On teste si le PVE répond
    $connection = @fsockopen($PVE_IP, "8006", $errno, $errstr, 5);

    if ((! empty("$PVE_IP")) && (! empty("$uai")) && (is_resource($connection))) {

        try {
            $proxmox = new PVE("$PVE_IP", "root", $config['pve_password'], 8006, "pam", false);
        } catch (Exception $e) {
            return [
                "",
                "",
                ""
            ];
        }

        $result = ($proxmox->nodes()
            ->node("pve-$uai")
            ->Storage()
            ->storage($config['pbs_host'])
            ->content()
            ->get())['data'];

        if ($result) {
            foreach ($result as $cle => $valeur) {
                if ((key_exists('notes', $result[$cle])) && ($result[$cle]['notes'] == "se4fs-$uai")) {
                    $SE4FS_SVG[] = $result[$cle];
                }
            }
            if (is_array($SE4FS_SVG)) {
                usort($SE4FS_SVG, 'compareCtime');
                $NbSVG = count($SE4FS_SVG);
                $LastSVG = array_pop($SE4FS_SVG)['ctime'];
                $sizeSVG = array_pop($SE4FS_SVG)['size'];

                return [
                    $NbSVG,
                    $LastSVG,
                    $sizeSVG
                ];
            } else {
                return [
                    "",
                    "",
                    ""
                ];
            }
        } else {
            return [
                "",
                "",
                ""
            ];
        }
    } else {
        return [
            "",
            "",
            ""
        ];
    }
}

function compareCtime($a, $b)
{
    return $a['ctime'] > $b['ctime'];
}

function AllStatsSauvegardes($config)
{
    $success = false;
    apcu_fetch('AllStatsSVG', $success);
    if ($success) {
        return apcu_fetch('AllStatsSVG');
    } else {
        $ServersConfArray = ServersConfArray();
        foreach ($ServersConfArray as $cle => $valeur) {
            if ($valeur['etat'] == "actif") {
                $AllStats[$valeur['uai']] = Sauvegardes($config, $valeur['uai']);
            }
        }
        apcu_add('AllStatsSVG', $AllStats, 3600);
        return $AllStats;
    }
}

?>