<?php
/**
 * Query influxdb
 */
require_once ('/var/www/sambaedu/vendor/autoload.php');
use InfluxDB2\Client;
use InfluxDB2\Model\WritePrecision;
use InfluxDB2\Point;

function StatsServeur($config, $quoi = "CPU", $host = "all", $duree = "5", $max = false)
{
    // Mise en cache des résultats de Query sur l'influxDB
    // On met le resultat dans une variable qui est un hash
    // des paramètres de la fonction
    $queryHash = hash('sha256', $quoi . $host . $duree . $max);
    $success = false;
    $query = apcu_fetch("StatsServeur-$queryHash", $success);

    if ($success) {
        return $query;
    }

    $start = "-" . $duree . "d";
    if ($quoi == "CPU") {
        // start: -24h)
        $measure = "cpu";
        $field = "cpuload";
    } elseif ($quoi == "nbUsersActivated") {
        $measure = "users";
        $field = "nbUsersActivated";
    } elseif ($quoi == "Users") {
        $measure = "users";
        $field = "nbUsers";
    } elseif ($quoi == "NbUsersConnected") {
        $measure = "users";
        $field = "nbusersconnected";
    } elseif ($quoi == "disks") {
        $measure = "disks";
        $field = "disks";
    } elseif (($quoi == "se4ad_ip") || ($quoi == "se4fs_ip")) {
        $measure = "IP";
        $field = $quoi;
    } else {
        $measure = "ram";
        $field = "ramused";
    }
    $QueryResult = InfluxQuery($config, $measure, $field, $host, $start, false, $max);
    apcu_add("StatsServeur-$queryHash", $QueryResult, 295);
    return $QueryResult;
}

function InfluxQuery($config, $measure, $field, $host, $start = "-1h", $last = false, $max = false)
{
    $maxquery = '';
    $movingAverage = "";
    $aggregate = "";
    $lastq = "";
    if ($max) {
        $aggregate = "|> aggregateWindow(every: 10m, fn: mean, createEmpty: false)";
        $maxquery = '|> max()';
    } elseif ($last) {
        $lastq = "|> last()";
    } else {
        $duree = "10";
        $average = 5;
        $movingAverage = '|> movingAverage(n:' . $average . ')';
        $aggregate = "|> aggregateWindow(every: " . $duree . "m, fn: mean, createEmpty: false)";
    }

    if ($field == "disks") {
        $fields = "|> filter(fn: (r) => r[\"_field\"] == \"home\")";
    } elseif ($host == $config['pbs_host']) {
        $fields = "|> filter(fn: (r) => r[\"datastore\"] == \"" . $config['pbs_datastore'] . "\")
    |> filter(fn: (r) => r[\"_field\"] == \"$field\")";
    } else {
        $fields = "|> filter(fn: (r) => r[\"_field\"] == \"$field\")";
    }
    if ($host == "all" && $field == "nbusersconnected") {
        $hostFilter = "|> group(columns: [\"_time\", \"_measurement\"])
        |> sum()
        |> group()";
        $movingAverage = "";
    } else {
        $hostFilter = "|> filter(fn: (r) => r[\"host\"] == \"$host\")";
    }

    $client = new Client([
        "url" => $config['influxdb_url'],
        "token" => $config['influxdb_token']
    ]);
    $queryApi = $client->createQueryApi();

    $query = "from(bucket: \"" . $config['influxdb_bucket'] . "\")
    |> range(start: " . $start . ")
    " . $lastq . "
    |> filter(fn: (r) => r[\"_measurement\"] == \"$measure\")
    " . $fields . "
    " . $aggregate . "
    " . $hostFilter . "
    " . $movingAverage . "
    " . $maxquery . "
    |> yield(name: \"mean\")";

    if ($last || $max) {
        $tables = $queryApi->query($query, $config['influxdb_org']);
        foreach ($tables as $table) {
            foreach ($table->records as $record) {
                if (isset($record->values) && is_object($record->values)) {
                    $records[$record] = $record->values;
                }
            }
        }
        if (isset($record)) {
            $value = round($record->values['_value'], 2) ?? "0";
            $time = $record->values['_time'] ?? "0";
            return [
                $value,
                $time
            ];
        } else {
            return [
                0,
                0
            ];
        }
    } else {
        $records = $queryApi->queryRaw($query, $config['influxdb_org']);
        return $records;
    }
}

/*
 * Retourne un tableau avec les derniers enregistrements des UAI
 * pour toutes les données stockées dans le json
 * accélère grandement le temps de traitement
 */
function allLastStats($config)
{
    $success = false;
    $last = apcu_fetch('allLastStats', $success);
    if (! $success) {

        $client = new Client([
            "url" => $config['influxdb_url'],
            "token" => $config['influxdb_token']
        ]);
        $queryApi = $client->createQueryApi();

        $start = "-1000h";
        $query = "from(bucket: \"" . $config['influxdb_bucket'] . "\")
    |> range(start: " . $start . ")
    |> last()
    |> filter(fn: (r) => r[\"_measurement\"] == \"disks\" or r[\"_measurement\"] == \"cpu\" or r[\"_measurement\"] == \"ram\" or r[\"_measurement\"] == \"IP\" or r[\"_measurement\"] == \"users\" or r[\"_measurement\"] == \"computers\" or r[\"_measurement\"] == \"nbusersconnected\")
    |> yield(name: \"mean\")";

        $tables = $queryApi->query($query, $config['influxdb_org']);

        $last = array();
        foreach ($tables as $table) {
            foreach ($table->records as $record) {
                $uai = $record->values['host'];
                $last[$uai][$record->values['_field']] = $record->values['_value'];
                // @TODO alive n'est pas toujours défini
                if (! isset($alive[$uai])) {
                    $last[$uai]['alive'] = strtotime($record->values['_time']);
                    /*
                     * les stats remontent les users (validés et totaux) même lorsque le se4 n'est pas actif ...
                     * il faut vérifier que le serveur est bien légitimement
                     * repéré comme actif
                     */
                    if (! array_key_exists("cpuload", $last[$uai])) {
                        $last[$uai]['alive'] = 0;
                    }
                }
            }
        }

        apcu_add('allLastStats', $last, 500);
    }

    return $last;
}

function maxUserOnline($config, $host = "all")
{
    $success = false;
    $maxUserOnline = apcu_fetch('maxUserOnline_' . $host, $success);
    if (! $success) {
        $maxUserOnline = InfluxQuery($config, 'users', 'nbusersconnected', $host, $start = "-20d", $last = false, $max = true)[0] ?? 100;
        apcu_add('maxUserOnline_' . $host, $maxUserOnline, 600);
    }
    return $maxUserOnline;
}

?>